﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.Windows.Forms;

using WpkgInstaller;

namespace WpkgNotifier
{
    class TrayNotifier : ApplicationContext, IWpkgMessageCallbackHandlerContract
    {
        private NotifyIcon notifyIcon;
        private IWpkgMessageRequestHandlerContract messageChannel;
        private EventLog eventLog;
        private WpkgMessageState state;

        public TrayNotifier()
        {

            eventLog = new EventLog();
            string eventSourceName = "TrayNotifier";
            string logName = "WpkgNotifier";

            //TODO: need to create the event source at a point where we have admin rights
            if (!EventLog.SourceExists(eventSourceName))
            {
                EventLog.CreateEventSource(eventSourceName, logName);
            }

            eventLog.Source = eventSourceName;
            eventLog.Log = logName;
            
            OpenMessageChannel();
                
            MenuItem syncMenuItem = new MenuItem("Sync", new EventHandler(RequestSync));
            MenuItem queryMenuItem = new MenuItem("Query", new EventHandler(RequestQuery));
            MenuItem exitMenuItem = new MenuItem("Exit", new EventHandler(Exit));

            notifyIcon = new NotifyIcon();
            notifyIcon.Text = "WPKG notifier";
            notifyIcon.Icon = WpkgNotifier.Properties.Resources.wpkgIcon;
            notifyIcon.ContextMenu = new ContextMenu(new MenuItem[]
                { syncMenuItem, queryMenuItem, exitMenuItem });
            notifyIcon.Visible = true;
        }

        private void OpenMessageChannel()
        {
            eventLog.WriteEntry("Trying to open channel");
            try
            {
                string address = "net.pipe://localhost/www.ch.cam.ac.uk/WpkgNotifier";

                NetNamedPipeBinding binding = new NetNamedPipeBinding();
                InstanceContext context = new InstanceContext(this);
                DuplexChannelFactory<IWpkgMessageRequestHandlerContract> factory = new DuplexChannelFactory<IWpkgMessageRequestHandlerContract>(context, binding, address);

                messageChannel = factory.CreateChannel();
                eventLog.WriteEntry("channel created");
            }
            catch (Exception e)
            {
                eventLog.WriteEntry("problem creating channel");

                eventLog.WriteEntry(e.Message);
            }
        }

        void RequestSync(object sender, EventArgs e)
        {
            SetToolTip("Syncing");
            messageChannel.HandleMessage(WpkgMessageRequestType.DO_SYNC);
            SetToolTip("Done");
        }

        void RequestQuery(object sender, EventArgs e)
        {
            SetToolTip("Querying");
            messageChannel.HandleMessage(WpkgMessageRequestType.RUN_QUERY);
            SetToolTip("Done");
        }

        void Exit(object sender, EventArgs e)
        {
            // We must manually tidy up and remove the icon before we exit.
            // Otherwise it will be left behind until the user mouses over.
            notifyIcon.Visible = false;
            Application.Exit();
        }

        public void SetToolTip(string msg)
        {
            notifyIcon.Text = msg;
        }

        public void HandleResponse(WpkgMessageResponse response)
        {
            eventLog.WriteEntry(response.Message);
        }

        public void HandleMessage(WpkgMessageState status)
        {
            this.state = status;
            switch (status)
            {
                case WpkgMessageState.IDLE:
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.wpkgIcon;
                    break;
                case WpkgMessageState.SYNCING:
                case WpkgMessageState.QUERYING:
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.blue;
                    break;
            }
        }
    }
}
